/*
 * Biotix JavaScript Library, version 0.1.0a
 * Copyright (c) 2011 Björn Schulz
 *
 * Biotix is freely distributable under the terms of the MIT license.
 *
 */


var biotix = (function() {
    var biotix = {};
    
    
    biotix.version = {
        major: 0,
        minor: 1,
        patch: 0,
        status: 'alpha'
    };
    
    
    biotix.global = (function() {
        return this;
    })();
    
    
    biotix.global.biotix = biotix;
    
    
    biotix.keywords = [
        'extend',
        'statics',
        'mixins',
        'class',
        'base'
    ];
    
    
    biotix.define = function(data) {
        var cls = function() {
            return this.constructor.apply(this, arguments);
        };
        
        cls.statics = {};
        cls.mixins = {};
        cls.prototype.mixins = {};
        
        if (data.extend !== undefined) {
            cls.base = data.extend;
            cls.prototype = biotix.create(cls.base.prototype);
        }
        
        if (data.statics || cls.base && cls.base.statics) {
            biotix.update(cls.statics, cls.base.statics, data.statics);
        }
        
        biotix.keys(cls.statics).forEach(function(key) {
            cls[key] = cls.statics[key];
        });
        
        biotix.keys(data).forEach(function(key) {
            if (biotix.keywords.indexOf(key) === -1) {
                cls.prototype[key] = data[key];
            }
        });
        
        if (!(data).hasOwnProperty('constructor')) {
            cls.prototype.constructor = function() {};
        }
        
        cls.prototype.class = cls;
        
        biotix.mixin(cls, cls.base ? cls.base.mixins : {});
        biotix.mixin(cls, data.mixins);
        
        return cls;
    };
    
    
    biotix.mixin = function(cls, mixins) {
        biotix.update(cls.mixins, mixins);
        
        biotix.keys(mixins || {}).forEach(function(key) {
            var mixin = mixins[key];
            
            cls.prototype.mixins[key] = biotix.create(mixin);
            
            biotix.mixin(cls, mixin.mixins);
            
            biotix.keys(mixin.statics || {}).forEach(function(key) {
                if (!(key in cls.statics)) {
                    cls.statics[key] = mixin.statics[key];
                }
            });
            
            biotix.keys(mixin.prototype || {}).forEach(function(key) {
                if (!(key in cls.prototype)) {
                    cls.prototype[key] = mixin.prototype[key];
                }
            });
        });
        
        return cls;
    };
    
    
    biotix.update = function(destination) {
        if (destination === undefined || destination === null) {
            return {};
        }
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            if (source !== undefined && source !== null) {
                biotix.keys(source).forEach(function(key) {
                    destination[key] = source[key];
                });
            }
        }
        return destination;
    };
    
    
    biotix.clone = function(object) {
        biotix.update({}, object);
    };
    
    
    biotix.create = function(cls) {
        if (cls.hasOwnProperty('prototype')) {
            return biotix.global.Object.create(cls.prototype);
        }
        return biotix.global.Object.create(cls);
    };
    
    
    biotix.keys = function(object) {
        return biotix.global.Object.keys(object);
    };
    
    
    biotix.Object = biotix.define({});
    
    
    return biotix;
})();


if (typeof module !== 'undefined' && module.exports !== undefined) {
    module.exports = biotix;
}